package statistics_service

import (
	"XT_New/models"
	"XT_New/service"
	"fmt"
	"github.com/jinzhu/gorm"
	"strconv"
)

func GetOrgFirstPatientInfo(user_org_id int64) (patient *models.Patients, err error) {
	db := service.XTReadDB()
	var tempPatient models.Patients
	findErr := db.Model(&models.Patients{}).Where("user_org_id = ? AND status = 1 AND lapseto = 1", user_org_id).First(&tempPatient).Error
	if findErr == gorm.ErrRecordNotFound {
		return nil, nil
	}
	if findErr != nil {
		return nil, findErr
	}
	return &tempPatient, nil
}

type ProcessIndexDataStruct struct {
	Date  string  `json:"date"`
	Value float64 `json:"value"`
}

func GetDialysisProcessIndexChartData(user_org_id int64, patient_id int64, start_time int64, end_time int64, statistics_type int) (datas []*ProcessIndexDataStruct, err error) {
	db := service.XTReadDB()
	switch statistics_type {
	case 1:
		err = db.Raw("select before.assessment_date as date, before.weighing_before as value from xt_assessment_before_dislysis as before Where status = 1 AND user_org_id = ? AND patient_id = ?  created_time <= ? AND  created_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 2:
		err = db.Raw("select after.assessment_date as date, after.weight_after as value from xt_assessment_after_dislysis as after Where status = 1 AND user_org_id = ? AND patient_id = ?  created_time <= ? AND  created_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error

		break
	case 3:
		err = db.Raw("select before.assessment_date as date, before.systolic_blood_pressure as value from xt_assessment_before_dislysis as before Where status = 1 AND user_org_id = ? AND patient_id = ?  created_time <= ? AND  created_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error

		break
	case 4:
		err = db.Raw("select before.assessment_date as date, before.diastolic_blood_pressure as value from xt_assessment_before_dislysis as before Where status = 1 AND user_org_id = ? AND patient_id = ?  created_time <= ? AND  created_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error

		break
	case 5:
		err = db.Raw("select before.assessment_date as date, before.pulse_frequency as value from xt_assessment_before_dislysis as before Where status = 1 AND user_org_id = ? AND patient_id = ?  created_time <= ? AND  created_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error

		break
	case 6:
		err = db.Raw("select after.assessment_date as date, after.systolic_blood_pressure as value from xt_assessment_after_dislysis as after Where status = 1 AND user_org_id = ? AND patient_id = ?  created_time <= ? AND  created_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 7:
		err = db.Raw("select after.assessment_date as date, after.diastolic_blood_pressure as value from xt_assessment_after_dislysis as after Where status = 1 AND user_org_id = ? AND patient_id = ?  created_time <= ? AND  created_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error

		break
	case 8:
		err = db.Raw("select after.assessment_date as date, after.pulse_frequency as value from xt_assessment_after_dislysis as after Where status = 1 AND user_org_id = ? AND patient_id = ?  created_time <= ? AND  created_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break

	}
	if err != nil {
		return nil, err
	}
	return datas, nil

}

type otherItemAmount struct {
	Total int64   `json:"total"`
	Name  string  `json:"name"`
	Ratio float32 `json:"ratio"`
}

func GetDialysisProcessIndexOtherChartData(user_org_id int64, patient_id int64, start_time int64, end_time int64, statistics_type int) (item []*otherItemAmount, err error) {
	db := service.XTReadDB()
	var items []*otherItemAmount
	var tempErr error
	var total int64
	switch statistics_type {
	case 9:
		tempErr = db.Table("xt_dialysis_prescription as p").Where("p.user_org_id=? and p.status=1 and  p.created_time >= ? and p.created_time <= ? AND p.patient_id = ? ", user_org_id, start_time, end_time, patient_id).
			Select("CASE WHEN p.mode == 1 THEN 'HD'" +
				" WHEN p.mode == 2 THEN 'HDF'" +
				" WHEN p.mode == 3 THEN 'HD+HP'" +
				" WHEN p.mode == 4 THEN 'HP'" +
				" WHEN p.mode == 5 THEN 'HF'" +
				" WHEN p.mode == 6 THEN 'SCUF'" +
				" WHEN p.mode == 7 THEN 'IUF'" +
				" WHEN p.mode == 8 THEN 'HFHD'" +
				" WHEN p.mode == 9 THEN 'HFHD+HP'" +
				" WHEN p.mode == 10 THEN 'PHF'" +
				" WHEN p.mode == 11 THEN 'HFR'" +
				" WHEN p.mode == 12 THEN 'HDF+HP'" +
				" WHEN p.mode == 13 THEN 'CRRT'" +
				" WHEN p.mode == 14 THEN '腹水回输'" +
				" WHEN p.mode == 19 THEN 'IUF+HD'" +
				" ELSE '未知' END AS name, COUNT(*) AS total",
			).Group("p.mode").Scan(&items).Error

		break
	case 10:
		config, _ := GetAnticoagulantData(user_org_id)
		selectContent := "CASE"
		for _, item := range config {
			selectContent = selectContent + " WHERE p.anticoagulant ==" + strconv.Itoa(item.Value) + " THEN " + item.Name

		}
		fmt.Println(selectContent)
		tempErr = db.Table("xt_dialysis_prescription as p").Where("p.user_org_id=? and p.status=1 and  p.created_time >= ? and p.created_time <= ? AND p.patient_id = ? ", user_org_id, start_time, end_time, patient_id).
			Select(selectContent +
				" ELSE '未知' END AS name, COUNT(*) AS total",
			).Group("p.mode").Scan(&items).Error

		break
	}

	db.Table("xt_dialysis_prescription").Where("p.user_org_id=? AND p.status = 1 AND p.patient_id = ? AND p.created_time >= ? and p.created_time <= ? ", user_org_id, patient_id, start_time, end_time).Count(&total)
	for _, item := range items {
		item.Ratio = float32(item.Total/total) * 100
	}
	if tempErr != nil {

		return nil, err
	}
	return items, nil
}

type PatientDialysisRecord struct {
	models.DialysisOrder
	DeviceNumber            string                         `gorm:"-" json:"device_number" form:"device_number"`
	DialysisPrescription    VMDialysisPrescription         `json:"prescription" gorm:"foreignkey:RecordDate;AssociationForeignKey:DialysisDate;"`
	PredialysisEvaluation   VMPredialysisEvaluation        `json:"predialysis_evaluation" gorm:"foreignkey:AssessmentDate;AssociationForeignKey:DialysisDate;"`
	AssessmentAfterDislysis models.AssessmentAfterDislysis `json:"assessment_after_dislysis" gorm:"foreignkey:AssessmentDate;AssociationForeignKey:DialysisDate;"`
}

func GetDialysisProcessIndexTableData(orgID, patientID int64, page, limit, start, end int64) ([]*PatientDialysisRecord, int64, error) {
	offset := (page - 1) * limit
	var total int64
	var err error
	var orders []*PatientDialysisRecord
	readDb := service.XTReadDB()
	db := readDb.Table("xt_dialysis_order as do").
		Preload("DialysisPrescription", "patient_id=? and user_org_id=? and status=1", patientID, orgID).
		Preload("PredialysisEvaluation", "patient_id=? and user_org_id=? and status=1", patientID, orgID).
		Preload("AssessmentAfterDislysis", "patient_id=? and user_org_id=? and status=1", patientID, orgID).
		Joins("JOIN xt_schedule as s ON s.patient_id=? and FROM_UNIXTIME(s.schedule_date, '%Y-%m-%d')=FROM_UNIXTIME(do.dialysis_date, '%Y-%m-%d')", patientID).
		Joins("JOIN xt_device_number as dm ON dm.org_id = ? and dm.id= do.bed_id", orgID).
		Where("do.patient_id=? and do.user_org_id=? and do.stage = 2 and do.status=1", patientID, orgID).Group("s.schedule_date")
	if start != 0 {
		db = db.Where("do.dialysis_date>=?", start)
	}
	if end != 0 {
		db = db.Where("do.dialysis_date<=?", end)
	}
	err = db.Count(&total).Offset(offset).Limit(limit).Order("do.dialysis_date desc").Select(" do.id, do.dialysis_date, do.stage , dm.name as device_number").Find(&orders).Error
	return orders, total, err
}

func GetAnticoagulantData(user_org_id int64) (config []*models.Dataconfig, tempErr error) {
	var tempDataConfig []*models.Dataconfig
	var tempConfig models.Dataconfig
	db := service.XTReadDB()
	db.Model(&models.Dataconfig{}).Where("name = 抗凝剂 AND field_name = anticoagulant AND parent_id = 0").First(&tempConfig)
	err := db.Model(&models.Dataconfig{}).Where("(org_id = 0 OR org_id = ?) AND parent_id = ?", user_org_id, tempConfig.ID).Find(&tempDataConfig).Error
	return tempDataConfig, err
}

//透析监测模块
func GetDialysisWatchChartData(user_org_id int64, patient_id int64, start_time int64, end_time int64, statistics_type int) (datas []*ProcessIndexDataStruct, err error) {
	db := service.XTReadDB()
	switch statistics_type {
	case 1:
		err = db.Raw("select monitor.operate_time as date, monitor.systolic_blood_pressure as value from xt_monitoring_record as monitor Where status = 1 AND user_org_id = ? AND patient_id = ?  operate_time <= ? AND  operate_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 2:
		err = db.Raw("select monitor.operate_time as date, monitor.diastolic_blood_pressure as value from xt_monitoring_record as monitor Where status = 1 AND user_org_id = ? AND patient_id = ?  operate_time <= ? AND  operate_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 3:
		err = db.Raw("select monitor.operate_time as date, monitor.pulse_frequency as value from xt_monitoring_record as monitor Where status = 1 AND user_org_id = ? AND patient_id = ?  operate_time <= ? AND  operate_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 4:
		err = db.Raw("select monitor.operate_time as date, monitor.breathing_rate as value from xt_monitoring_record as monitor Where status = 1 AND user_org_id = ? AND patient_id = ?  operate_time <= ? AND  operate_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 5:
		err = db.Raw("select monitor.operate_time as date, monitor.temperature as value from xt_monitoring_record as monitor Where status = 1 AND user_org_id = ? AND patient_id = ?  operate_time <= ? AND  operate_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 6:
		err = db.Raw("select monitor.operate_time as date, monitor.blood_flow_volume as value from xt_monitoring_record as monitor Where status = 1 AND user_org_id = ? AND patient_id = ?  operate_time <= ? AND  operate_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 7:
		err = db.Raw("select monitor.operate_time as date, monitor.venous_pressure as value from xt_monitoring_record as monitor Where status = 1 AND user_org_id = ? AND patient_id = ?  operate_time <= ? AND  operate_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 8:
		err = db.Raw("select monitor.operate_time as date, monitor.arterial_pressure as value from xt_monitoring_record as monitor Where status = 1 AND user_org_id = ? AND patient_id = ?  operate_time <= ? AND  operate_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	case 9:
		err = db.Raw("select monitor.operate_time as date, monitor.ultrafiltration_rate as value from xt_monitoring_record as monitor Where status = 1 AND user_org_id = ? AND patient_id = ?  operate_time <= ? AND  operate_time >= ? ", user_org_id, patient_id, end_time, start_time).Scan(&datas).Error
		break
	}
	if err != nil {
		return nil, err
	}
	return datas, nil

}
func GetDialysisWatchTableData(orgID, patientID int64, page, limit, start, end int64) ([]*models.MonitoringRecord, int64, error) {
	offset := (page - 1) * limit
	var total int64
	var err error
	var orders []*models.MonitoringRecord
	readDb := service.XTReadDB()
	db := readDb.Table("xt_monitoring_record as mo").
		Where("mo.patient_id=? and mo.user_org_id=?  and mo.status=1", patientID, orgID)
	if start != 0 {
		db = db.Where("mo.operate_time>=?", start)
	}
	if end != 0 {
		db = db.Where("mo.operate_time<=?", end)
	}
	err = db.Count(&total).Offset(offset).Limit(limit).Order("mo.operate_time desc").Select("mo.systolic_blood_pressure, mo.diastolic_blood_pressure, mo.pulse_frequency,mo.breathing_rate,mo.temperature,mo.blood_flow_volume,mo.venous_pressure,mo.arterial_pressure,mo.transmembrane_pressure,mo.dialysate_temperature,mo.ultrafiltration_rate,mo.ultrafiltration_volume").Find(&orders).Error
	return orders, total, err
}

//患者体重统计

func GetPatientWeightChartData(user_org_id int64, patient_id int64, start_time int64, end_time int64, statistics_type int) (item []*otherItemAmount, err error) {
	db := service.XTReadDB()
	var items []*otherItemAmount
	var tempErr error
	//var total int64
	switch statistics_type {
	case 1:
		tempErr = db.Table("xt_assessment_before_dislysis as before").Where("before.user_org_id=? and before.status=1 and  before.created_time >= ? and before.created_time <= ?", user_org_id, start_time, end_time).
			Select("CASE WHEN before.dry_weight < 40 THEN '小于40kg'" +
				" WHEN before.dry_weight >= 40 AND before.dry_weight < 50 THEN '40~50kg'" +
				" WHEN before.dry_weight >= 50 AND before.dry_weight < 60 THEN '50~60kg'" +
				" WHEN before.dry_weight >= 60 AND before.dry_weight < 70 THEN '60~70kg'" +
				" WHEN before.dry_weight >= 70 THEN '大于70kg'" +
				" ELSE '未知' END AS name, COUNT(*) AS total",
			).Scan(&items).Error

		break
	case 2:
		tempErr = db.Table("xt_assessment_before_dislysis as before").Where("before.user_org_id=? and before.status=1 and  before.created_time >= ? and before.created_time <= ?", user_org_id, start_time, end_time).
			Select("CASE WHEN before.weighing_before < 40 THEN '小于40kg'" +
				" WHEN before.weighing_before >= 40 AND before.weighing_before < 50 THEN '40~50kg'" +
				" WHEN before.weighing_before >= 50 AND before.weighing_before < 60 THEN '50~60kg'" +
				" WHEN before.weighing_before >= 60 AND before.weighing_before < 70 THEN '60~70kg'" +
				" WHEN before.weighing_before >= 70 THEN '大于70kg'" +
				" ELSE '未知' END AS name, COUNT(*) AS total",
			).Scan(&items).Error
		break
	case 3:

		break
	case 4:
		tempErr = db.Table("xt_assessment_after_dislysis as after").Where("after.user_org_id=? and after.status=1 and  after.created_time >= ? and after.created_time <= ?", user_org_id, start_time, end_time).
			Select("CASE WHEN after.weight_after < 40 THEN '小于40kg'" +
				" WHEN after.weight_after >= 40 AND after.weight_after < 50 THEN '40~50kg'" +
				" WHEN after.weight_after >= 50 AND after.weight_after < 60 THEN '50~60kg'" +
				" WHEN after.weight_after >= 60 AND after.weight_after < 70 THEN '60~70kg'" +
				" WHEN after.weight_after >= 70 THEN '大于70kg'" +
				" ELSE '未知' END AS name, COUNT(*) AS total",
			).Scan(&items).Error
		break
	}

	//db.Table("xt_dialysis_prescription").Where("p.user_org_id=? AND p.status = 1 AND p.patient_id = ? AND p.created_time >= ? and p.created_time <= ? ", user_org_id, patient_id,start_time,end_time).Count(&total)
	//for _, item := range items{
	//	item.Ratio = float32(item.Total / total) * 100
	//}
	if tempErr != nil {

		return nil, err
	}
	return items, nil
}
